local addonInfo, privateVars = ...

if nkQuestTracker ~= nil then return end
nkQuestTracker = {}

privateVars.internal = {}
privateVars.uiElements = {}

privateVars.uiElements.context = UI.CreateContext("nkQuestTracker")
privateVars.uiElements.context:SetStrata ('dialog')
privateVars.uiElements.context:SetLayer(2)

privateVars.uiElements.secureContext = UI.CreateContext("nkQuestTracker.Secure")
privateVars.uiElements.secureContext:SetSecureMode('restricted')
privateVars.uiElements.secureContext:SetStrata ('layout')
privateVars.uiElements.secureContext:SetLayer(3)

privateVars.uiElements.configContext = UI.CreateContext ("nkQuestTracker.Config")
privateVars.uiElements.configContext:SetStrata ('dialog')
privateVars.uiElements.configContext:SetLayer(2)

privateVars.uiElements.tooltipContext = UI.CreateContext("nkQuestTrackerTooltip")
privateVars.uiElements.tooltipContext:SetStrata ('topmost')
privateVars.uiElements.tooltipContext:SetLayer(999)

privateVars.questCache = {}
privateVars.useItemUpdates = {}
privateVars.internalMove = {}
privateVars.questItemList = {}
privateVars.useButtonQuestItemID = nil
privateVars.returnQuestItem = nil

privateVars.craftingItems = {
	"I4CD48A0656A66436,41331FF662BDB8ED,,,,,,",
	"I21D9052625C52295,709CF1BA97DA8BF9,,,,,,",
	"I2D199E412D5BD46E,DEFE32C2157DE5B1,,,,,,"
}

privateVars.updateQuestList = {}
privateVars.addQuestList = {}
privateVars.removeQuestList = {}
privateVars.update = false
privateVars.zoneInfo = {}

privateVars.zoneFilter = false
privateVars.playerZoneID = nil

privateVars.addonInit = false

privateVars.playerAvailable = false

function privateVars.internal.main(_, addon)

	if addon == addonInfo.identifier then
	
		table.insert(Command.Slash.Register("nkQuestTracker"), {privateVars.internal.commandHandler, "nkQuestTracker", "config"})
	
		local items = {
		{ label = privateVars.langTexts.buttonMenuConfig, callBack = function () nkQuestTracker.ShowConfig() end},
		{  label = privateVars.langTexts.buttonMenuToggle, callBack = function () nkQuestTracker.Toggle() end},
		}

		if nkPanel ~= nil and nkQuestTrackerSetup.nkPanel == true then
			privateVars.uiElements.panel = privateVars.internal.nkPanelPlugin(items)
			nkPanel.api.registerPlugin('nkQuestTracker', privateVars.uiElements.panel)					
		else			
			EnKai.manager.init('nkQuestTracker', items, nil)			
		end
		
		nkQuestBase.loadPackage("classic")
		nkQuestBase.loadPackage("nt")
		
		EnKai.version.init(addonInfo.toc.Identifier, addonInfo.toc.Version)			
		
		Command.Event.Attach(Event.Unit.Availability.Full, privateVars.internal.UnitAvailable, "nkQuestTracker.Unit.Availability.Full")
	end
	
end

function privateVars.internal.UnitAvailable (_, units)

	if privateVars.playerUNID == nil then
		local playerDetails = Inspect.Unit.Detail('player')
		if playerDetails ~= nil then
			privateVars.playerUNID = playerDetails.id
			privateVars.playerZoneID = playerDetails.zone
			privateVars.playerLevel = playerDetails.level
		end
	else
		local playerDetails = Inspect.Unit.Detail('player')
		if playerDetails ~= nil then 
			privateVars.playerZoneID = playerDetails.zone
			privateVars.playerLevel = playerDetails.level
		end	
	end
	
	if privateVars.playerUNID == nil then return end	
	if units[privateVars.playerUNID] == nil then return end
	
	privateVars.playerAvailable = true	
	
	EnKai.inventory.init()

	if privateVars.uiElements.questLog == nil then
		-- only first time
		privateVars.uiElements.questLog = privateVars.internal.buildUI ()
				
		privateVars.uiElements.craftingList = privateVars.internal.craftingUI()
		privateVars.uiElements.missingList = privateVars.internal.missingUI()
		
		privateVars.internal.fillLog ()
		
		privateVars.uiElements.progressBar = EnKai.uiCreateFrame("nkProgressBar", "nkQuestTracker.progressBar", privateVars.uiElements.questLog)
		privateVars.uiElements.progressBar:SetPoint("CENTERTOP", privateVars.uiElements.questLog, "CENTERTOP", 0, 40)
		
		privateVars.uiElements.progressBar:SetWidth(privateVars.uiElements.questLog:GetWidth()-20)
		privateVars.uiElements.progressBar:SetHeight(20)
		privateVars.uiElements.progressBar:SetLayer(99)
		privateVars.uiElements.progressBar:SetVisible(false)
		privateVars.uiElements.progressBar:SetFontColor(0.925, 0.894, 0.741, 1)
		privateVars.uiElements.progressBar:SetBorderColor(0.925, 0.894, 0.741, 1)
		privateVars.uiElements.progressBar:SetFillColor(0.925, 0.894, 0.741, 1)
		
				
		Command.Event.Attach(Event.Quest.Accept, privateVars.internal.questAcceptEvent, "nkQuestTracker.Quest.Accept")
		Command.Event.Attach(Event.Quest.Abandon, privateVars.internal.questAbandonEvent, "nkQuestTracker.Quest.Abandon")
		Command.Event.Attach(Event.Quest.Change, privateVars.internal.questChangeEvent, "nkQuestTracker.Quest.Change")
		Command.Event.Attach(Event.Quest.Complete, privateVars.internal.questCompleteEvent, "nkQuestTracker.Quest.Complete")
		Command.Event.Attach(Event.System.Update.Begin, privateVars.internal.systemUpdate, "nkQuestTracker.System.Update.Begin")
	
		Command.Event.Attach(Event.Unit.Availability.None, privateVars.internal.unitNotAvaiable, "nkQuestTracker.Unit.Availability.None")
		Command.Event.Attach(Event.Unit.Availability.Partial, privateVars.internal.unitNotAvaiable, "nkQuestTracker.Unit.Availability.None")
		
		Command.Event.Attach(Event.Unit.Detail.Level, privateVars.internal.unitLevel, "nkQuestTracker.Unit.Detail.Level")
		
		
		Command.Event.Attach(Event.Item.Slot, privateVars.internal.SlotUpdate, "nkQuestTracker.inventory.Item.Slot")
		Command.Event.Attach(Event.Item.Update, privateVars.internal.SlotUpdate, "nkQuestTracker.inventory.Item.Update")
	end

	privateVars.uiElements.questLog:SetTitle(addonInfo.name)
	--Command.Event.Detach(Event.Unit.Availability.Full, nil, "nkQuestTracker.Unit.Availability.Full")	

end

function privateVars.internal.commandHandler(commandline)

	nkQuestTracker.ShowConfig()
	
end

function privateVars.internal.settingsHandler(_, addon) 

	if addon == addonInfo.identifier then
	
		if nkQuestTrackerSetup == nil then 
			nkQuestTrackerSetup = { nkPanel = true, xpos = 800, ypos = 300, width = 300, height = 500, useXpos = 800, useYpos = 100, useUI = true,
										showLevel = true, showZone = true, sortBy = 'level', colorByLevel = true, autoHide = false,
								moveable = true, displayHeader = true,  bgAlpha = .6, categoryHeaderSize = 16,
								categoryOrder = { "crafting", "daily", "weekly", "guild", "ia", "pvp", "world", "zone", "area", "instant", "raid", "personal", "carnage"},
								categoryColor = { 
													guild = { 0, 0.58, 1 }, ia = { 0.737, 1, 0.804 }, pvp = { 1, 0.417, 0 },
													world = {1, 1, 1}, zone = { 0.737, 1, 0.804 }, area = { 0.655, 1, 0.357 }, instant = { 1, 0.847, 0 }, raid = { 1, 0.417, 0 },
												  crafting = { 0, 0.715, 0.875 }, daily = { 0.498, 0.788, 1 }, weekly = { 0, 0.58, 1 }, personal = { 0.839, 0.498, 1}, carnage = { 0.839, 0.498, 1}},
								categoryShow = {crafting = true, world = true, daily = true, guild = true, ia = true, weekly = true, zone = true, area = true, instant = true, raid = true, personal = true, carnage = true, pvp = true},
								categoryFontSize = { 
														ia = { header = 15, body = 13},  guild = { header = 15, body = 13},  carnage = { header = 15, body = 13}, pvp = { header = 15, body = 13},
														world = { header = 15, body = 13},  zone = { header = 15, body = 13},  area = { header = 15, body = 13},
													 instant = { header = 15, body = 13}, raid = { header = 15, body = 13}, personal = { header = 15, body = 13},
													 crafting = { header = 15, body = 13}, daily = { header = 15, body = 13}, weekly = { header = 15, body = 13}},
								bodyColor = {1, 1, 1},
								bodyCompleteColor = {.6, .6, .6},
								collapseState = {},
								categoryCollapseState = {}
						 } 
		else
		
			if nkQuestTrackerSetup.categoryHeaderSize == nil then nkQuestTrackerSetup.categoryHeaderSize = 16 end
			
			if nkQuestTrackerSetup.categoryCollapseState == nil then nkQuestTrackerSetup.categoryCollapseState = {} end
			
			if nkQuestTrackerSetup.useXpos == nil then nkQuestTrackerSetup.useXpos = 800 end
			if nkQuestTrackerSetup.useYpos == nil then nkQuestTrackerSetup.useYpos = 100 end
			if nkQuestTrackerSetup.useUI == nil then nkQuestTrackerSetup.useUI = true end
			
			if nkQuestTrackerSetup.autoHide == nil then nkQuestTrackerSetup.autoHide = false end
			
			if nkQuestTrackerSetup.categoryColor.pvp == nil then
        table.insert(nkQuestTrackerSetup.categoryOrder, 6, "pvp")
        nkQuestTrackerSetup.categoryColor.pvp = { 0, 0.58, 1 }
        nkQuestTrackerSetup.categoryShow.pvp = true
        nkQuestTrackerSetup.categoryFontSize.pvp = { header = 15, body = 13}
			end
			
			if nkQuestTrackerSetup.categoryColor.guild == nil then
				table.insert(nkQuestTrackerSetup.categoryOrder, 4, "guild")
				table.insert(nkQuestTrackerSetup.categoryOrder, 5, "ia")
				
				nkQuestTrackerSetup.categoryColor.guild = { 0, 0.58, 1 }
				nkQuestTrackerSetup.categoryColor.ia = { 0.737, 1, 0.804 }
				
				nkQuestTrackerSetup.categoryShow.guild = true
				nkQuestTrackerSetup.categoryShow.ia = true
				
				nkQuestTrackerSetup.categoryFontSize.guild = { header = 15, body = 13}
				nkQuestTrackerSetup.categoryFontSize.ia = { header = 15, body = 13}
				
			end
			
			if nkQuestTrackerSetup.showLevel == nil then nkQuestTrackerSetup.showLevel = true end
			if nkQuestTrackerSetup.showZone == nil then nkQuestTrackerSetup.showZone = true end
			if nkQuestTrackerSetup.sortBy == nil then nkQuestTrackerSetup.sortBy = 'name' end
			if nkQuestTrackerSetup.colorByLevel == nil then nkQuestTrackerSetup.colorByLevel = true end
			
			-- V2.0.1 changes
			
			if nkQuestTrackerSetup.categoryColor["carnage"] == nil then
			   table.insert(nkQuestTrackerSetup.categoryOrder, "carnage")
			   nkQuestTrackerSetup.categoryColor["carnage"] = { 0.839, 0.498, 1}
			   nkQuestTrackerSetup.categoryShow["carnage"] = true
			   nkQuestTrackerSetup.categoryFontSize["carnage"] = { header = 15, body = 13}
			end
			
		end
			
	end
					
end

Command.Event.Attach(Event.Addon.Load.End, privateVars.internal.main, "nkQuestTracker.Addon.Load.End")
Command.Event.Attach(Event.Addon.SavedVariables.Load.End, privateVars.internal.settingsHandler, "nkQuestTracker.SavedVariables.Load.End")

function privateVars.internal.SlotUpdate(_, updates)

  if nkQuestTrackerSetup.useUI == false then return end

--  if privateVars.returnQuestItem ~= nil then return end
  
	for slot, key in pairs(updates) do
		if EnKai.strings.startsWith(slot, "sqst") then
			if privateVars.internalMove[slot] == true then
				privateVars.internalMove[slot] = nil
			else
				if key == false then
					--if privateVars.questItemList[key] == true then					
					--	privateVars.uiElements.useUI:Update()
					--end
				else
					local details = Inspect.Item.Detail(key)
					local questItem = nkQuestBase.query.questItemByKey (details.type)
					if questItem ~= nil then
						privateVars.uiElements.useUI:Update()
					end
				end
			end
		end
	end

end

--------------------------------------------------------------------------------------------------------------

function privateVars.internal.unitNotAvaiable(_, units)

	if privateVars.playerUNID == nil then return end	
	if units[privateVars.playerUNID] == nil then return end
	
	privateVars.playerAvailable = false
	
	if privateVars.uiElements.questLog ~= nil then
		privateVars.uiElements.questLog:SetTitle(addonInfo.name .. " (PAUSED)")
	end

end

function privateVars.internal.unitLevel(_, units)

	if privateVars.playerAvailable == false then return end
	if units[privateVars.playerUNID] == nil then return end	
	if units[privateVars.playerUNID] == false then return end
	
	privateVars.playerLevel = units[privateVars.playerUNID]
	if nkQuestTrackerSetup.colorByLevel == true then
		privateVars.internal.clearLog(privateVars.internal.fillLog)
	end

end

function privateVars.internal.systemUpdate()

	if privateVars.playerAvailable == false then return end
	if nkQuestBase.query.isInit() == false then return end
		
	if privateVars.uiElements.useUI == nil and Inspect.System.Secure() == false then
		if EnKai.inventory.getAvailableSlots() ~= false then				
			privateVars.uiElements.useUI = privateVars.internal.buildUseUI ()
			privateVars.uiElements.useUI:Update()
			--privateVars.uiElements.useUI:SetVisible(nkQuestTrackerSetup.useUI)
		end
	elseif Inspect.System.Secure() == false and privateVars.useUpdate == true then
		privateVars.uiElements.useUI:Update()
		privateVars.useUpdate = false
	end
	
	if Inspect.System.Secure() == false and #privateVars.useItemUpdates > 0 then
		for k, v in pairs(privateVars.useItemUpdates) do
			if v.type == 'add' then			
				privateVars.uiElements.useUI:AddUseItem(v.key, v.name, v.icon)
			else
				privateVars.uiElements.useUI:RemoveUseItem(v.key)
			end
		end
		privateVars.useItemUpdates = {}
	end	
	
	if privateVars.update == true then return end
	
	if privateVars.forceUpdate ~= true then
		if privateVars.lastQuestUpdate == nil then
      	privateVars.lastQuestUpdate = Inspect.Time.Real()
      	privateVars.forceUpdate = true
    	else
      	local tmpTime = Inspect.Time.Real()
      	if EnKai.tools.math.round((tmpTime - privateVars.lastQuestUpdate), 1) > 1 then privateVars.forceUpdate = true end
    	end
  	end
  
  	if privateVars.forceUpdate == true then
  	
  		if #privateVars.addQuestList > 0 then

  			privateVars.internal.questAdd(privateVars.addQuestList)
  			privateVars.addQuestList = {}
  			
      elseif #privateVars.updateQuestList > 0 then
      
        	privateVars.internal.questChange(privateVars.updateQuestList)
  			privateVars.updateQuestList = {}

		elseif #privateVars.removeQuestList > 0 then
		
			privateVars.internal.questRemove(privateVars.removeQuestList)
  			privateVars.removeQuestList = {}
			
		end

      privateVars.lastQuestUpdate = Inspect.Time.Real()
      privateVars.forceUpdate = false
	end
end

function privateVars.internal.isUpdate(cached, details)
        
	local isUpdate = false
	
	--print ('is update ' .. details.name)
        
	if details.name ~= cached.name or details.complete ~= cached.complete or #details.objective ~= #cached.objective then
		isUpdate = true
   elseif details.objective == nil or cached.objective == nil then
     	isUpdate = true
	else
   	for idx = 1, #details.objective, 1 do
   		if details.objective[idx] ~= nil and cached.objective[idx] ~= nil then
	      	if details.objective[idx].complete ~= cached.objective[idx].complete then
	      		isUpdate = true
	      	elseif details.objective[idx].count ~= cached.objective[idx].count then
	      		isUpdate = true
	      	elseif details.objective[idx].description ~= cached.objective[idx].description then
	      		isUpdate = true
	      	elseif details.objective[idx].countDone ~= cached.objective[idx].countDone then
	            isUpdate = true
				end
			end
		end
	end
	
	return isUpdate

end

--------------------------------------------------------------------------------------------------------------

function privateVars.internal.questAdd(list)
	
	local addCoRoutine = coroutine.create(
		function ()

			for idx = 1, #list, 1 do
				privateVars.uiElements.progressBar:SetValue(idx)
				local key = list[idx]
				if key ~= nil then
  				local flag, details = pcall(Inspect.Quest.Detail, key)
  				if flag then 
    				privateVars.internal.processQuest(details, true)				
    		  		privateVars.questCache[key] = details
    		  		
    		  		privateVars.uiElements.questLog:AddQuest(key, details.domain, details.name, details.objective, details.complete, details.level, details.zone)
    		  		coroutine.yield(idx)
    		  	end
    		 end
  		 end
		end
	)
	
	local callBack = function ()
		privateVars.update = false
		if (privateVars.addonInit == false) then
		   privateVars.uiElements.questLog:GetContent():SetVisible(true)
		   privateVars.addonInit = true
		end
		privateVars.uiElements.progressBar:SetVisible(false)
		if privateVars.uiElements.panel ~= nil then privateVars.uiElements.panel:UpdateTitle() end
	end
	
	privateVars.uiElements.progressBar:SetRange(1, #list)
	privateVars.uiElements.progressBar:SetValue(1)
	if #list > 3 and privateVars.uiElements.questLog:GetContent():GetVisible() == true then privateVars.uiElements.progressBar:SetVisible(true) end
	EnKai.coroutines.add ({ func = addCoRoutine, counter = #list, active = true, callBack = callBack })
	privateVars.update = true

end

function privateVars.internal.questChange(list)

	local changeCoRoutine = coroutine.create(
		function ()

			for idx = 1, #list, 1 do
				privateVars.uiElements.progressBar:SetValue(idx)
				local key = list[idx]
				local flag, details = pcall( Inspect.Quest.Detail, key)
				if flag then 
  				privateVars.internal.processQuest(details, true)				
  		  		
  		  		if privateVars.questCache[key] ~= nil and details.name ~= "" then
  		  			local err, isUpdate = pcall(privateVars.internal.isUpdate, privateVars.questCache[key], details)
  		  			
  		  			if err == false then
  						if nkQuestTrackerSetup.debug == true then
  							print('---------------------------------')
  							dump(privateVars.questCache[key])
  							dump(details)
  						end	
  
  						list[key] = nil
  					elseif err == true and isUpdate == true then
  						privateVars.uiElements.questLog:UpdateQuest(key, details.domain, details.name, details.objective, details.complete, details.level)
            		end
        		end
          
              privateVars.questCache[key] = details
  		  		
  		  		coroutine.yield(idx)
  		    end
		  	end
		end
	)
	
	local callBack = function ()
		privateVars.update = false
		privateVars.uiElements.progressBar:SetVisible(false)
		if privateVars.uiElements.panel ~= nil then privateVars.uiElements.panel:UpdateTitle() end
	end
	
	privateVars.uiElements.progressBar:SetRange(1, #list)
	privateVars.uiElements.progressBar:SetValue(1)
	if #list > 3 and privateVars.uiElements.questLog:GetContent():GetVisible() == true then privateVars.uiElements.progressBar:SetVisible(true) end
	EnKai.coroutines.add ({ func = changeCoRoutine, counter = #list, active = true, callBack = callBack })
	privateVars.update = true
	
end

function privateVars.internal.questRemove(list)
	
	local removeCoRoutine = coroutine.create(
		function ()

			for idx = 1, #list, 1 do
				privateVars.uiElements.progressBar:SetValue(idx)
				
				if type(list[idx]) == 'function' then
					list[idx]()
				else
					local key = list[idx]
					privateVars.uiElements.questLog:RemoveQuest(key)
					privateVars.questCache[key] = nil
				end
				
		  		coroutine.yield(idx)
		  	end
		end
	)
	
	local callBack = function ()
		privateVars.update = false
		privateVars.uiElements.progressBar:SetVisible(false)
		if privateVars.uiElements.panel ~= nil then privateVars.uiElements.panel:UpdateTitle() end
		privateVars.uiElements.useUI:Update()
	end
	
	privateVars.uiElements.progressBar:SetRange(1, #list)
	privateVars.uiElements.progressBar:SetValue(1)
	if #list > 3 and privateVars.uiElements.questLog:GetContent():GetVisible() == true then privateVars.uiElements.progressBar:SetVisible(true) end
	EnKai.coroutines.add ({ func = removeCoRoutine, counter = #list, active = true, callBack = callBack })
	privateVars.update = true

end

--------------------------------------------------------------------------------------------------------------

function privateVars.internal.clearLog(callBack)

	local list = Inspect.Quest.List()

	for key, v in pairs(list) do
		table.insert(privateVars.removeQuestList, key)
	end
	
	if callBack ~= nil then table.insert(privateVars.removeQuestList, callBack) end

end

function privateVars.internal.fillLog ()

	privateVars.uiElements.questLog:GetContent():SetVisible(false)

	local list = Inspect.Quest.List()
	local flag, details = pcall(Inspect.Quest.Detail, list)
	
	if flag == false then return end
	
	local newCollapseState = {}
	
	local questList = { personal = {} }
	
	for k, v in pairs(details) do
		if v.name ~= nil and v.name ~= "" then 	
			local addQuest = true
			if privateVars.zoneFilter == true then
				addQuest = nkQuestBase.query.IsQuestInZone (privateVars.playerZoneID, v.id)
			end
		
			if addQuest == true then
				table.insert(privateVars.addQuestList, k)
				newCollapseState[k] = nkQuestTrackerSetup.collapseState[k]
			end
		end
	end
	
	nkQuestTrackerSetup.collapseState = newCollapseState
	
	if (#privateVars.addQuestList == 0) then privateVars.uiElements.questLog:GetContent():SetVisible(true) end
	 
end

function privateVars.internal.questAcceptEvent (_, quests)

	for k, v in pairs (quests) do
      table.insert(privateVars.addQuestList, k)
	end
	
end

function privateVars.internal.questAbandonEvent (_, quests)

	for k, v in pairs (quests) do
      table.insert(privateVars.removeQuestList, k)
	end

end

function privateVars.internal.questChangeEvent (_, quests)

  for k, v in pairs (quests) do
      table.insert(privateVars.updateQuestList, k)
  end

end

function privateVars.internal.questCompleteEvent (_, quests)

	for k, v in pairs (quests) do
      table.insert(privateVars.removeQuestList, k)
	end

end

function privateVars.internal.processQuest(details, processTitleFlag)

	if details.rewardChoose ~= nil then
		for k, v in pairs(details.rewardChoose) do
		
			if EnKai.tools.table.isMember (privateVars.craftingItems, k) == true then
				if details.tagName ~= nil then details.name = string.format("%s (%s)", details.name, details.tagName) end
				details.domain = "crafting"
				break
			end
		end
	end
	
	if details.rewardGuaranteed ~= nil then		
		for k, v in pairs(details.rewardGuaranteed) do
		
			if EnKai.tools.table.isMember (privateVars.craftingItems, k) == true then
				if details.tagName ~= nil then details.name = string.format("%s (%s)", details.name, details.tagName) end
				details.domain = "crafting"
				break
			end
		end
	end
	
	if details.domain ~= 'crafting' and details.tag ~= nil and string.find(details.tag, 'weekly') ~= nil then
		if processTitleFlag == true then details.name = string.format("%s (%s)", details.name, details.tagName) end
		details.domain = 'weekly'
	elseif details.domain ~= 'crafting' and details.tag ~= nil and string.find(details.tag, 'daily') ~= nil then
		if processTitleFlag == true then details.name = string.format("%s (%s)", details.name, details.tagName) end
		details.domain = 'daily'
	elseif details.domain == nil then
		details.domain = 'personal'
	end
	
	local lvl, libDetails = nkQuestBase.query.byKey(details.id, nkQuestTrackerSetup.showZone)
	
	if libDetails ~= nil then
	
		if libDetails.domain ~= nil then 
		  details.domain = libDetails.domain
		elseif libDetails.type ~= nil then
       if EnKai.tools.table.isMember(libDetails.type, 9) then
          details.domain = "carnage"
       end
		end				
		
		details.grp = libDetails.grp
		if nkQuestTrackerSetup.showLevel == true then details.level = lvl end
		details.use = libDetails.use
		if nkQuestTrackerSetup.showZone == true and libDetails.zoneId ~= nil and libDetails.zoneId ~= "UKNOWN_ZONE" then
			if privateVars.zoneInfo[libDetails.zoneId] == nil then
				local zoneInfo = Inspect.Zone.Detail(libDetails.zoneId)
				if zoneInfo ~= nil then
					privateVars.zoneInfo[libDetails.zoneId] = zoneInfo.name
				end
			end
			
			if privateVars.zoneInfo[libDetails.zoneId] ~= nil then
				details.zone = privateVars.zoneInfo[libDetails.zoneId]
			end 
		end
	end
	
	if processTitleFlag == true then
	
		if nkQuestTrackerSetup.colorByLevel == true then
			local color = "4FC012"
			
			if lvl == nil then
				color = "D4C200"
			elseif lvl < privateVars.playerLevel -7 then
				color = "8E8E8E"
			elseif lvl > privateVars.playerLevel +5 then
				color = "C51212"
			elseif lvl > privateVars.playerLevel + 2 then
				color = "D4C200"
			end
			
			if color ~= nil then
				details.name = string.format("<font color='#%s'>%s</font>", color, details.name)
			end
			
		end
	
		if details.complete == true then details.name = string.format(privateVars.langTexts.completeInfo, details.name) end
		
		if details.zone ~= nil and details.level ~= nil then
			if nkQuestTrackerSetup.sortBy == 'name' then
				details.name = string.format("%s [%s] [%s]", details.name, details.level, details.zone)
			elseif nkQuestTrackerSetup.sortBy == 'zone' then
				details.name = string.format("[%s] [%s] %s", details.zone, details.level, details.name )
			else
				details.name = string.format("[%s] [%s] %s", details.level, details.zone, details.name )
			end
		elseif details.zone ~= nil then
			if nkQuestTrackerSetup.sortBy == 'name' then
				details.name = string.format("%s [%s]", details.name, details.zone)
			elseif details.zone ~= nil then
				details.name = string.format("[%s] %s", details.zone, details.name )
			end
		else	
			if nkQuestTrackerSetup.sortBy == 'name' then
				details.name = string.format("%s [%s]", details.name, details.level)
			elseif details.level ~= nil then	
				details.name = string.format("[%s] %s", details.level, details.name )
			end	
		end
	end

	return lvl
	
end

--------------------------------------------------------------------------------------------------------------

function privateVars.internal.showMenu (parent, key)

	privateVars.questMenuKey = key

	if privateVars.uiElements.menu == nil then
		privateVars.uiElements.menu = EnKai.uiCreateFrame("nkMenu", 'nkQuestTracker.menu', privateVars.uiElements.context)
		privateVars.uiElements.menu:SetLayer(3)
		privateVars.uiElements.menu:AddEntry({ label = privateVars.langTexts.track, callBack = privateVars.internal.trackQuest })
		privateVars.uiElements.menu:AddEntry({ label = privateVars.langTexts.abandon, callBack = privateVars.internal.abandonQuest })
		privateVars.uiElements.menu:AddEntry({ label = privateVars.langTexts.share, callBack = privateVars.internal.shareQuest})
	end
	
	local menu = privateVars.uiElements.menu
	local mouse = Inspect.Mouse()
	
	menu:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouse.x, mouse.y)
	menu:SetVisible(true)
	
end

function privateVars.internal.showMenuCategory (parent, category)

  privateVars.categoryMenuCategory = category

  if privateVars.uiElements.menuCategory == nil then
    privateVars.uiElements.menuCategory = EnKai.uiCreateFrame("nkMenu", 'nkQuestTracker.menuCategory', privateVars.uiElements.context)
    privateVars.uiElements.menuCategory:SetLayer(3)
    privateVars.uiElements.menuCategory:AddEntry({ label = privateVars.langTexts.abandonAll, callBack = privateVars.internal.abandonQuestCategory })
  end
  
  local menuCategory = privateVars.uiElements.menuCategory
  local mouse = Inspect.Mouse()
  
  menuCategory:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouse.x, mouse.y)
  menuCategory:SetVisible(true)  

end

function privateVars.internal.trackQuest()

  pcall (Command.Quest.Track, privateVars.questMenuKey)	 
	privateVars.uiElements.menu:SetVisible(false)

end

function privateVars.internal.abandonQuest ()

	local function noFunc ()
		privateVars.uiElements.menu:SetVisible(false)
	end
	
	local function yesFunc()
	  pcall (Command.Quest.Abandon, privateVars.questMenuKey) 
		privateVars.uiElements.menu:SetVisible(false)
	end
	
	local flag, quest = pcall(Inspect.Quest.Detail, privateVars.questMenuKey)
	if flag == false then return end
	
	local text = string.format(privateVars.langTexts.abandonQuestConfirm, quest.name)

	privateVars.internal.confirmDialog(text, yesFunc, noFunc) 
	
end

function privateVars.internal.shareQuest ()

	Command.Quest.Share(privateVars.questMenuKey)
	privateVars.uiElements.menu:SetVisible(false)
	
end

function privateVars.internal.abandonQuestCategory ()

  local function noFunc ()
    privateVars.uiElements.menuCategory:SetVisible(false)
  end
  
  local function yesFunc()
  
    local uiCategory = privateVars.uiElements.questLog:GetCategory(privateVars.categoryMenuCategory)
    local questList = uiCategory:GetQuestList()
    
    for k, v in pairs(questList) do
      Command.Quest.Abandon(v)
    end
     
    privateVars.uiElements.menuCategory:SetVisible(false)
  end
  
  local text = string.format(privateVars.langTexts.abandonAllQuestsConfirm, privateVars.langTexts.showCategoryCheckbox[privateVars.categoryMenuCategory])

  privateVars.internal.confirmDialog(text, yesFunc, noFunc) 

end

--------------------------------------------------------------------------------------------------------------

function privateVars.internal.showTooltip (parent, questkey, itemkey, category, message)

	if privateVars.uiElements.tooltip == nil then
		privateVars.uiElements.tooltip = EnKai.uiCreateFrame("nkTooltip", 'nkQuestTracker.tooltip', privateVars.uiElements.tooltipContext)
		privateVars.uiElements.tooltip:SetLayer(2)
	end
	
	local tooltip = privateVars.uiElements.tooltip
	local quest, flag
	tooltip:ClearAll()
	tooltip:SetTitleFontSize(15)
	
	if message ~= nil then
		tooltip:SetTitle(addonInfo.toc.Identifier)
		tooltip:SetTitleColor(1, 1, 1)
	else
	   if (questkey ~= nil) then
		   flag, quest = pcall(Inspect.Quest.Detail, questkey)		  
		   if flag == false or quest == nil then return end
		   if quest.domain == nil then quest.domain = 'personal' end
		   
		   tooltip:SetTitle(quest.name)
		elseif itemkey ~= nil then
		   
		   local details = Inspect.Item.Detail(itemkey)
		   if (details == nil) then return end
		   
		   local dbDetails = nkQuestBase.query.questItemByKey(details.type)
		   if dbDetails ~= nil and dbDetails['use_' .. EnKai.tools.lang.getLanguageShort()] ~= nil then
		      message = dbDetails['use_' .. EnKai.tools.lang.getLanguageShort()]
		   elseif details.flavor ~= nil then
          message = details.flavor
        else
          message = ""
        end

	      tooltip:SetTitle(details.name)
		   
		end
		
		local color = nkQuestTrackerSetup.categoryColor[category]
		if color == nil then color = {1, 1, 1} end
		tooltip:SetTitleColor(color[1], color[2], color[3])
	end
				
	local text = ""
	if message ~= nil then
		text = message
	elseif quest.summary ~= nil then
		text = quest.summary
	elseif quest.description ~= nil then
		text = quest.description
	end
	
	local lines = {{ text = text, wordwrap = true, minWidth = 200, fontsize = 13 }}
	
	local lvl, libDetails = nil, nil
	
	if (questkey ~= nil ) then
	 lvl, libDetails = nkQuestBase.query.byKey(questkey, nkQuestTrackerSetup.showZone)
	end
	
	if libDetails ~= nil then
		local scene = libDetails.scene
		local npc
		
		if libDetails.giver ~= nil then
			npc = nkQuestBase.query.NPC (libDetails.giver)
			if npc ~= nil then
				if npc.scene ~= nil then scene = npc.scene[EnKai.tools.lang.getLanguageShort()] end
				table.insert (lines, { text = "", height = 10})
				table.insert (lines, { text = string.format(privateVars.langTexts.questGiver, npc[EnKai.tools.lang.getLanguageShort()]), wordwrap = true, fontsize=13})
			end
		end
	
		if scene ~= nil then
			local sceneInfo = EnKai.location.getSceneInfo(scene)
			if sceneInfo ~= nil then
				if sceneInfo[EnKai.tools.lang.getLanguageShort()] ~= nil then
					scene = sceneInfo[EnKai.tools.lang.getLanguageShort()]
				end
			end
			
			if npc == nil then table.insert (lines, { text = "", height = 10}) end
			table.insert (lines, { text = string.format(privateVars.langTexts.scene, scene), wordwrap = true, fontsize=13})
			
		end
		
		
	end
	
	if quest ~= nil and quest.objective ~= nil then
		table.insert (lines, { text = "", height = 10})
		
		for k, v in pairs(quest.objective) do
		
			local color = "#FFFFFF"
			if v.complete == true then color = "#DDDDDD" end
		
			table.insert(lines, {text = string.format("<font color='%s'>%s</font>", color, v.description), wordwrap = true, fontsize = 13 })
		end
	end
	
	tooltip:SetLines(lines)
	
	local mouse = Inspect.Mouse()
	
	if mouse.x + tooltip:GetWidth() > UIParent:GetWidth() then
	 tooltip:SetPoint("TOPRIGHT", UIParent, "TOPRIGHT", -(UIParent:GetWidth()-mouse.x), mouse.y)
	else
	 tooltip:SetPoint("TOPLEFT", UIParent, "TOPLEFT", mouse.x, mouse.y)
	end
		
	tooltip:SetVisible(true)

end

--------------------------------------------------------------------------------------------------------------

function nkQuestTracker.ShowConfig()

	if privateVars.uiElements.config == nil then
		privateVars.uiElements.config = nkQuestTracker.Config ()
	end
	
	privateVars.uiElements.config:SetVisible(true) 

end

function nkQuestTracker.Toggle()

  if privateVars.uiElements.questLog ~= nil then
    if privateVars.uiElements.questLog:GetVisible() == true then
      privateVars.uiElements.questLog:getUseItemButton():SetVisible(false)
      privateVars.uiElements.questLog:SetVisible(false)      
    else
      privateVars.uiElements.questLog:getUseItemButton():SetVisible(true)
      privateVars.uiElements.questLog:SetVisible(true)
    end    
  end

end


--------------------------------------------------------------------------------------------------------------

function privateVars.internal.confirmDialog (message, yesFunc, noFunc)

	local name = "nkQuestTracker.confirmDialog"

	if privateVars.uiElements.confirmDialog == nil then
		privateVars.uiElements.confirmDialog = EnKai.uiCreateFrame("nkDialog", name, privateVars.uiElements.context)
		privateVars.uiElements.confirmDialog:SetWidth(500)
		privateVars.uiElements.confirmDialog:SetHeight(250)
		privateVars.uiElements.confirmDialog:SetType('confirm')
		privateVars.uiElements.confirmDialog:SetLayer(99)
	end
	
	privateVars.uiElements.confirmDialog:SetMessage(message)
	privateVars.uiElements.confirmDialog:SetVisible(true)
	
	Command.Event.Detach(EnKai.events[name].LeftButtonClicked, nil, name .. ".LeftButtonClicked") -- delete possible earlier event handler and replace with new one
	Command.Event.Detach(EnKai.events[name].RightButtonClicked, nil, name .. ".RightButtonClicked") -- delete possible earlier event handler and replace with new one
		
	Command.Event.Attach(EnKai.events[name].LeftButtonClicked, function ()
		yesFunc()
	end, name .. ".LeftButtonClicked")
	
	if noFunc ~= nil then	
		Command.Event.Attach(EnKai.events[name].RightButtonClicked, function ()
			noFunc()
		end, name .. ".RightButtonClicked")
	end
		
end


function privateVars.internal.messageDialog (message)

	if privateVars.uiElements.messageDialog == nil then
		privateVars.uiElements.messageDialog = EnKai.uiCreateFrame("nkDialog", "nkQuestTracker.messageDialog", privateVars.uiElements.configContext)
		privateVars.uiElements.messageDialog:SetWidth(500)
		privateVars.uiElements.messageDialog:SetHeight(200)
		privateVars.uiElements.messageDialog:SetType('message')
		privateVars.uiElements.messageDialog:SetLayer(99)
	end
	
	privateVars.uiElements.messageDialog:SetMessage(message)
	privateVars.uiElements.messageDialog:SetVisible(true)

end
