local addonInfo, privateVars = ...

function privateVars.internal.missingList ()

	if privateVars.uiElements.missingList:GetVisible() == true then 
		privateVars.uiElements.missingList:SetVisible(false)
		return
	end
	
	local playerDetails = Inspect.Unit.Detail('player')
	
	local list = nkQuestBase.query.getQuestsByZone(playerDetails.zone)
	if list == nil then
		privateVars.uiElements.missingList:SetVisible(false)
		return
	end
	
	privateVars.uiElements.craftingList:SetVisible(false)

  if Inspect.System.Secure() == false then Command.System.Watchdog.Quiet() end
	
	local complete = Inspect.Quest.Complete()
	
  local questList = {}

  for _, questId in pairs (list) do
    table.insert(questList, questId)   
  end 

  local tempList = {}
  local queryList = {}		
  local count = 0
  
  privateVars.uiElements.progressBar:SetRange(1, #questList)
  privateVars.uiElements.progressBar:SetValue(1)
  privateVars.uiElements.progressBar:SetVisible(true)
  
  local gridValues = {}
  
	local missingCoRoutine = coroutine.create(
    function ()
      for idx = 1, #questList, 1 do
        
        privateVars.uiElements.progressBar:SetValue(idx)

        local questId = questList[idx]
        if complete[questId] == nil then
          local abbrev = string.format("%sxxxxxxxx", string.sub(questId, 1, 8))
          if complete[abbrev] == nil then
            local lvl, libDetails = nkQuestBase.query.byKey(questId, false)
            if libDetails ~= nil and libDetails.domain ~= "ia" then
                
              local detailsList = Inspect.Quest.Detail({[questId]=true})
              for key, details in pairs(detailsList) do   
                if details.domain ~= 'ia' then
                  local level = privateVars.internal.processQuest(details, false)
                  local domainText = privateVars.langTexts.showCategoryCheckbox[details.domain]
                  table.insert(gridValues, { {key = key, value = details.name}, { value = level }, { value = domainText } })
                end           
              end 
            end       
          end
        end   
          
        coroutine.yield(idx)
      end
    end
  )

  local callBack = function ()
    privateVars.uiElements.progressBar:SetVisible(false) 
    table.sort (gridValues, function (v1, v2) return v1[1].value < v2[1].value end )

    local grid = privateVars.uiElements.missingList:GetGrid() 
    
    grid:SetRowPos(1, false)
    grid:SetCellValues(gridValues)
    privateVars.uiElements.missingList:SetVisible(true)
    
    if #gridValues > 30 then
      privateVars.uiElements.missingList:GetSlider():SetVisible(true)
      privateVars.uiElements.missingList:GetSlider():SetRange(1, #gridValues-29)
      privateVars.uiElements.missingList:GetSlider():AdjustValue(1)
    else
      privateVars.uiElements.missingList:GetSlider():SetVisible(false)
    end 
  end
  
  EnKai.coroutines.add ({ func = missingCoRoutine, counter = #questList, active = true, callBack = callBack })
	
end

function privateVars.internal.missingUI ()
	
	local name = "nkQuestTracker.missingUI"
	local gridRows = math.floor(privateVars.uiElements.questLog:GetHeight() / 19)
	local slider
	
	local ui = UI.CreateFrame("Frame", name, privateVars.uiElements.questLog)
	ui:SetWidth(400)
	ui:SetHeight(privateVars.uiElements.questLog:GetHeight())
	ui:SetPoint("TOPRIGHT", privateVars.uiElements.questLog, "TOPLEFT")
	ui:SetBackgroundColor(0, 0, 0, nkQuestTrackerSetup.bgAlpha)
	ui:SetVisible(false)	
	
	local grid = EnKai.uiCreateFrame("nkGrid", name .. 'grid', ui)
		
	grid:SetHeaderHeight(0)
	grid:SetPoint("TOPLEFT", ui, "TOPLEFT")
	grid:SetBorderColor(0, 0, 0, nkQuestTrackerSetup.bgAlpha)
	
	if nkQuestTrackerSetup.bgAlpha-0.2 < 0 then
    grid:SetBodyColor(0, 0, 0, 0)
	else
	  grid:SetBodyColor(0, 0, 0, nkQuestTrackerSetup.bgAlpha-0.2)
	end
	
	grid:SetTransparentHeader()
		
	local cols = {	{ align = 'left', width = 300},
						{ align = 'center', width = 25},
						{ align = 'center', width = 75}  }	
	
	grid:Layout (cols, gridRows)
	
	Command.Event.Attach(EnKai.events[name .. 'grid'].MouseOver, function (_, rowPos)
		local ttRowPos = grid:GetRowPos() + rowPos -1
		local key = grid:GetKey(ttRowPos)
		if key ~= nil then privateVars.internal.showTooltip(grid:GetCell(rowPos, 1), key, nil, nil, nil) end
	end, name .. 'grid.Grid.MouseOver')
	
	Command.Event.Attach(EnKai.events[name .. 'grid'].MouseOut, function (_, rowPos)			
		if privateVars.uiElements.tooltip ~= nil then privateVars.uiElements.tooltip:SetVisible(false) end
	end, name .. 'grid.Grid.MouseOut')
	
	Command.Event.Attach(EnKai.events[name .. 'grid'].WheelForward, function (_, rowPos)			
		slider:AdjustValue (rowPos)
	end, name .. 'grid.Grid.WheelForward')
	
	Command.Event.Attach(EnKai.events[name .. 'grid'].WheelBack, function (_, rowPos)
		slider:AdjustValue (rowPos)
	end, name .. 'grid.Grid.WheelBack')
	
	slider = EnKai.uiCreateFrame("nkScrollbox", name .. "slider", grid)	
		
	slider:SetPoint("TOPLEFT", grid, "TOPRIGHT", -14, 0)
	slider:SetHeight(grid:GetHeight())
	slider:SetRange(1, 1)
	slider:SetVisible(false)
	slider:SetLayer(2)
	slider:SetColor( 1,1,1,1 )
	
	Command.Event.Attach(EnKai.events[name .. "slider"].ScrollboxChanged, function ()		
		grid:SetRowPos(math.floor(slider:GetValue('value')), true)
	end, name .. 'slider.ScrollboxChanged')
	
	function ui:GetGrid() return grid end
	function ui:GetSlider() return slider end

	return ui	

end

function privateVars.internal.markQuestOnMap (key)

end